/* Copyright (C) 1993, 1995, 1996 Free Software Foundation, Inc.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3, or (at your option)
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/* This is almost copied from strncpy.c, written by Torbjorn Granlund.  */

#ifndef _STPNCPY_H_
#define _STPNCPY_H_

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#ifndef PARAMS
# if PROTOTYPES || (defined (__STDC__) && __STDC__)
#  define PARAMS(args) args
# else
#  define PARAMS(args) ()
# endif
#endif

/* Copy no more than N characters of SRC to DEST, returning the address of
   the last character written into DEST.  */
char * stpncpy PARAMS ((char * dest, const char * src, size_t n));

#endif
